#!/bin/sh

module=drweb

check_os_release() {
    os=unknown

    if [ -f /etc/redhat-release ]; then
        if  grep -q '^Red Hat Enterprise Linux Server release 5' \
            /etc/redhat-release || \
            grep -q '^CentOS release 5' /etc/redhat-release; then
            os=rhel5
        elif grep -q '^Red Hat Enterprise Linux Server release 6' \
            /etc/redhat-release || \
            grep -q '^CentOS release 6' /etc/redhat-release; then
            os=rhel6
        fi
    fi

    if [ -f /etc/SuSE-release ]; then
        if grep -q '^SUSE Linux Enterprise Server 11' /etc/SuSE-release ; then
            os=sles11
        fi
    fi

    if [ -f /etc/astra_version ]; then
        if grep -q '^SE 1.6 (smolensk)' /etc/astra_version ; then
            os=astra_se16
        elif grep -q '^SE 1.5 (smolensk)' /etc/astra_version ; then
            os=astra_se15
        elif grep -q '^SE 1.4 (smolensk)' /etc/astra_version ; then
            os=astra_se14
        elif grep -q '^SE 1.3 (smolensk)' /etc/astra_version ; then
            os=astra_se13
        fi
    fi

    echo $os
}

check_release() {
    uname -r | grep -q "$1"
}

check_arch() {
    uname -m | grep -q "^$1$"
}

unsupported() {
    echo 'Warning: Your system does not support fanotify;' \
        'you need to build and install kernel module manually' \
        'from the source codes.'
}

install_mod() {
    from="/opt/drweb.com/lib/x86_64-linux-gnu/modules/$1/extra"
    to="/lib/modules/`uname -r`/extra"

    if [ ! -d "$from" ]; then
        echo "Directory \"$from\" not found."
        exit 1
    fi


    if check_arch mips; then
        :
    else
        while read x; do
            if ! grep -q "$x" /boot/System.map-`uname -r`; then
                unsupported
                return
            fi
        done < "$from/$module.ko.syms"
    fi

    module_path="/lib/modules/`uname -r`//extra/$module.ko"
    [ -f "$module_path" ] && mv "$module_path" "$module_path.drweb-old"

    mkdir -p "$to"
    ln -sf "$from/$module.ko" "$to"
    echo "$to/$module.ko" >> /var/opt/drweb.com/spider-kmod-symlinks

    echo "Installed \"$to/$module.ko\"."

    echo "Doing depmod ..."
    depmod

    if [ `uname -s` = Linux ] && \
        env LANG=C stat --printf %N /proc/1/exe | awk '{print $3}' | \
        grep /systemd >/dev/null; then
        drweb-ctl reload 2>/dev/null || true
    else
        /etc/init.d/drweb-configd reload || true
    fi

    if [ -n "$2" ]; then
        echo "$2"
    else
        echo 'Note: Your system does not support fanotify;' \
             'the supplied kernel module will be used.'
    fi
}

check_fanotify() {
    have_fanotify=

    if [ -f /proc/config.gz ]; then
        zcat /proc/config.gz | grep -q CONFIG_FANOTIFY=y && have_fanotify=1
    elif [ -f "/boot/config-`uname -r`" ]; then
        grep -q CONFIG_FANOTIFY=y "/boot/config-`uname -r`" && have_fanotify=1
    fi

    if [ -n "$have_fanotify" ]; then
        glibc=$(ldd /bin/cp 2>/dev/null | grep libc\\.so | \
            awk '{print $3}')
        if [ -f "$glibc" ]; then
            grep -q fanotify_init "$glibc" || have_fanotify=
        fi
    fi

    if [ -n "$have_fanotify" ]; then
        echo 'Note: Your system supports fanotify;' \
            'the kernel module is not required.'
        return 0
    fi

    return 1
}

check_fanotify && exit 0

if check_arch 'mips' ; then
    if check_release '^3\.19\.12-debug$' ; then
        install_mod 'mips/3.19.12-debug'
    else
        unsupported
    fi
    exit 0
fi

case `check_os_release` in
    astra_se13)
        check_release '^3\.2\.0-.*-generic$' && check_arch 'x86_64' && \
            install_mod 'x86_64/3.2.0-27-generic'
        ;;

    astra_se14)
        check_release '^3\.16\.0-.*-generic$' && check_arch 'x86_64' && \
            install_mod 'x86_64/3.16.0-16-generic'

        check_release '^3\.16\.0-.*-pax$' && check_arch 'x86_64' && \
            install_mod 'x86_64/3.16.0-16-pax'
        ;;

    astra_se15)
        check_release '^4\.2\.0-23-generic$' && check_arch 'x86_64' && \
            install_mod 'x86_64/4.2.0-23-generic'

        check_release '^4\.2\.0-24-generic$' && check_arch 'x86_64' && \
            install_mod 'x86_64/4.2.0-24-generic'

        check_release '^4\.2\.0-.*-pax$' && check_arch 'x86_64' && \
            install_mod 'x86_64/4.2.0-23-pax'
        ;;

    astra_se16)
        check_release '^4\.15\.3-1-hardened$' && check_arch 'x86_64' && \
            install_mod 'x86_64/4.15.3-1-hardened'

        check_release '^4\.15\.3-1-generic$' && check_arch 'x86_64' && \
            install_mod 'x86_64/4.15.3-1-generic'

        check_release '^4\.15\.3-2-hardened$' && check_arch 'x86_64' && \
            install_mod 'x86_64/4.15.3-2-hardened'

        check_release '^4\.15\.3-2-generic$' && check_arch 'x86_64' && \
            install_mod 'x86_64/4.15.3-2-generic'
        ;;
    rhel5)
        check_release '^2\.6\.18-.*el5$' && check_arch 'x86_64' && \
            install_mod 'x86_64/2.6.18-371.11.1.el5'

        check_release '^2\.6\.18-.*el5xen$' && check_arch 'x86_64' && \
                install_mod 'x86_64/2.6.18-371.11.1.el5xen'

        check_release '^2\.6\.18-.*el5$' && check_arch 'i686' && \
                install_mod 'i386/2.6.18-371.el5'

        check_release '^2\.6\.18-.*el5PAE$' && \
            install_mod 'i386/2.6.18-371.el5PAE'

        check_release '^2\.6\.18-.*el5xen$' && check_arch 'i686' && \
            install_mod 'i386/2.6.18-371.el5xen'
        ;;
    rhel6)
        check_release '^2\.6\.32-.*el6.i686$' && \
            install_mod 'i386/2.6.32-431.23.3.el6.i686'

        check_release '^2\.6\.32-.*el6.x86_64$' && \
            install_mod 'x86_64/2.6.32-431.23.3.el6.x86_64'
        ;;
    sles11)
        if check_arch 'i686' ; then
            pre='i386'
            check_release '^3\.0\.76.*default$' && \
                install_mod "$pre/3.0.76-0.11-default"

            check_release '^3\.0\.76.*pae' && \
                install_mod "$pre/3.0.76-0.11-pae"

            check_release '^3\.0\.76.*trace$' && \
                install_mod "$pre/3.0.76-0.11-trace"

            check_release '^3\.0\.76.*xen$' && \
                install_mod "$pre/3.0.76-0.11-xen"

            check_release '^3\.0\.101.*default$' && \
                install_mod "$pre/3.0.101-63-default"

            check_release '^3\.0\.101.*pae' && \
                install_mod "$pre/3.0.101-63-pae"

            check_release '^3\.0\.101.*trace$' && \
                install_mod "$pre/3.0.101-63-trace"

            check_release '^3\.0\.101.*xen$' && \
                install_mod "$pre/3.0.101-63-xen"
        elif check_arch 'x86_64' ; then
            pre='x86_64'
            check_release '^3\.0\.76.*default$' && \
                install_mod "$pre/3.0.76-0.11-default"

            check_release '^3\.0\.76.*trace$' && \
                install_mod "$pre/3.0.76-0.11-trace"

            check_release '^3\.0\.76.*xen$' && \
                install_mod "$pre/3.0.76-0.11-xen"

            check_release '^3\.0\.101.*default$' && \
                install_mod "$pre/3.0.101-63-default"

            check_release '^3\.0\.101.*trace$' && \
                install_mod "$pre/3.0.101-63-trace"

            check_release '^3\.0\.101.*xen$' && \
                install_mod "$pre/3.0.101-63-xen"
        fi
        ;;
    *)
        unsupported
        ;;
esac

exit 0
